<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://chalaksoft.ir
 * @since      1.0.0
 *
 * @package    Chalak_Hesap_Api
 * @subpackage Chalak_Hesap_Api/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Chalak_Hesap_Api
 * @subpackage Chalak_Hesap_Api/admin
 * @author     ChalakSoft dev <info@chalaksoft.ir>
 */
class Chalak_Hesap_Api_Admin
{

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string $plugin_name The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string $version The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @param string $plugin_name The name of this plugin.
     * @param string $version The version of this plugin.
     * @since    1.0.0
     */
    public function __construct($plugin_name, $version)
    {

        $this->plugin_name = $plugin_name;
        $this->version = $version;

        add_action('admin_menu', array('Chalak_Hesap_Api_Admin', 'menu'), 5);
    }

    public static function menu()
    {
        add_menu_page('chalkahesab_settigns', 'تنضیمات چالاک حساب', 'manage_options', 'chalkahesab_settigns', ['Chalak_Hesap_Api_Admin', 'show_setting_page']);
//        add_submenu_page( 'my-top-level-handle', 'Page title', 'Sub-menu title', 'manage_options', 'my-submenu-handle', 'my_magic_function');
    }

    public static function show_setting_page()
    {
        include_once __DIR__ . DIRECTORY_SEPARATOR . "page" . DIRECTORY_SEPARATOR . "setting.php";
    }


    /**
     * Register the stylesheets for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_styles()
    {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Chalak_Hesap_Api_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Chalak_Hesap_Api_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */

        wp_enqueue_style($this->plugin_name, plugin_dir_url(__FILE__) . 'css/chalak-hesap-api-admin.css', array(), $this->version, 'all');

    }

    /**
     * Register the JavaScript for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts()
    {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Chalak_Hesap_Api_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Chalak_Hesap_Api_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */

        wp_enqueue_script($this->plugin_name, plugin_dir_url(__FILE__) . 'js/chalak-hesap-api-admin.js', array('jquery'), $this->version, false);

    }

}
