<?php

class Chalak_Hesab_Api_ROUTES
{
    const token_key = 'chalak_hesab_token';

    //to save last ids

    const last_customer_id = 'chalak_hesab_last_customer_id';
    const last_product_id = 'chalak_hesab_last_product_id';

    const CHALAKHESAB_CHEKER_PARAM = 'sent_to_chalakhesab';

    public $namespace = 'chalak-hesab/v1';

    private function getLastProductId()
    {
        return get_option(self::last_product_id, 1);
    }

    private function setLastProductId($product_id)
    {
        return update_option(self::last_product_id, $product_id);
    }

    private function getLastCustomerId()
    {
        return get_option(self::last_customer_id, 1);
    }

    private function setLastCustomerId($customer_id)
    {
        return update_option(self::last_customer_id, $customer_id);
    }

    public function buildFields($array, $keys = null)
    {
        if (empty($keys)) {
            return $array;
        }

        return array_intersect_key($array, array_flip((array)$keys));
    }

    public function getFields($req, $key = null)
    {
        $fields = $req->get_param('fields');

        if (!is_null($key)) {
            $fields = $fields[$key] ?? [];
        }

        if (empty($fields)) {
            return [];
        }

        return explode(',', $fields);
    }

    /**
     * Register the routes for the objects of the controller.
     * @param $route WP_REST_Request
     */
    public function permissions_check($route)
    {
        if ($route->get_param('token') != get_option(self::token_key)) {
            return new WP_Error(403, 'your token is not valid');
        }

        return true;
    }

    /**
     * @param $route WP_REST_Request
     */
    public function check_token($route)
    {
        $token = $route->get_param('token');

        $data = [
            'status' => true,
            'message' => 'اطلاعات درست می باشد'
        ];

        if ($token != get_option(self::token_key)) {
            $data = [
                'status' => false,
                'message' => 'توکن درست نمی باشد'
            ];
        }

        return new WP_REST_Response($data, 200);
    }

    public function register_routes()
    {
        register_rest_route(
            $this->namespace,
            '/check-token',
            [[
                'methods' => \WP_REST_Server::READABLE,
                'callback' => [$this, 'check_token'],
            ]]
        );

        register_rest_route(
            $this->namespace,
            '/get-orders',
            [
                [
                    'methods' => \WP_REST_Server::READABLE,
                    'callback' => [$this, 'get_orders'],
                    'permission_callback' => [$this, 'permissions_check'],
                    [
                        'last_id' => [
                            'type' => 'int',
                            'description' => 'default is 1',
                            'required' => false,
                            'default' => 0,
                        ],
                        'page' => [
                            'type' => 'int',
                            'description' => 'default is 1',
                            'required' => false,
                        ],
                        'per_page' => [
                            'description' => 'default is 20',
                            'type' => 'int',
                            'required' => false,
                        ]
                    ],
                ]
            ]
        );

        register_rest_route(
            $this->namespace,
            '/get-order',
            [
                [
                    'args' => [
                        'order_id' => [
                            'description' => __('order id'),
                            'type' => 'int',
                            'required' => true,
                        ]
                    ],
                    'methods' => \WP_REST_Server::READABLE,
                    'callback' => [$this, 'get_order'],
                    'permission_callback' => [$this, 'permissions_check'],
                ]
            ]
        );

        register_rest_route(
            $this->namespace,
            '/get-customers',
            [
                [
                    [
                        'last_id' => [
                            'type' => 'int',
                            'description' => 'default is 1',
                            'required' => false,
                            'default' => 0,
                        ],
                        'page' => [
                            'type' => 'int',
                            'description' => 'default is 1',
                            'required' => false,
                        ],
                        'per_page' => [
                            'description' => 'default is 20',
                            'type' => 'int',
                            'required' => false,
                        ]
                    ],
                    'methods' => \WP_REST_Server::READABLE,
                    'callback' => [$this, 'get_customers'],
                    'permission_callback' => [$this, 'permissions_check'],
                ],
                'schema' => [$this, 'get_public_item_schema'],
            ]
        );

        register_rest_route(
            $this->namespace,
            '/get-customer',
            [
                [
                    'args' => [
                        'customer_id' => [
                            'description' => __('customer id'),
                            'type' => 'int',
                            'required' => true,
                        ]
                    ],

                    'methods' => \WP_REST_Server::READABLE,
                    'callback' => [$this, 'get_customer'],
                    'permission_callback' => [$this, 'permissions_check'],
                ],
                'schema' => [$this, 'get_public_item_schema'],
            ]
        );

        register_rest_route(
            $this->namespace,
            '/get-products',
            [
                [
                    'args' => [
                        'last_id' => [
                            'type' => 'int',
                            'description' => 'default is 1',
                            'required' => false,
                            'default' => 0,
                        ],
                        'page' => [
                            'type' => 'int',
                            'description' => 'default is 1',
                            'required' => false,
                        ],
                        'per_page' => [
                            'description' => 'default is 20',
                            'type' => 'int',
                            'required' => false,
                        ]
                    ],
                    'methods' => \WP_REST_Server::READABLE,
                    'callback' => [$this, 'get_products'],
                    'permission_callback' => [$this, 'permissions_check'],
                ],
                'schema' => [$this, 'get_public_item_schema'],
            ]
        );

        register_rest_route(
            $this->namespace,
            '/get-product',
            [
                [
                    'args' => [
                        'product_id' => [
                            'description' => __('product_id : product id '),
                            'type' => 'string',
                            'required' => true,
                        ]
                    ],
                    'methods' => \WP_REST_Server::READABLE,
                    'callback' => [$this, 'get_product'],
                    'permission_callback' => [$this, 'permissions_check'],
                ],
                'schema' => [$this, 'get_public_item_schema'],
            ]
        );

        register_rest_route(
            $this->namespace,
            '/product/update-qty',
            [
                [
                    'args' => [
                        'product_id' => [
                            'description' => __('id: product id'),
                            'type' => 'string',
                            'required' => true,
                        ],
                        'qty' => [
                            'description' => __('qty stock quantity'),
                            'type' => 'int',
                            'required' => true,
                        ]
                    ],
                    'methods' => \WP_REST_Server::READABLE,
                    'callback' => [$this, 'update_qty'],
                    'permission_callback' => [$this, 'permissions_check'],
                ],
                'schema' => [$this, 'get_public_item_schema'],
            ]
        );

        register_rest_route(
            $this->namespace,
            '/product/update-price',
            [
                [
                    'args' => [
                        'product_id' => [
                            'description' => __('id : product id'),
                            'type' => 'string',
                            'required' => true,
                        ],
                        'price' => [
                            'description' => __('product price'),
                            'type' => 'int',
                            'required' => true,
                        ]
                    ],
                    'methods' => \WP_REST_Server::READABLE,
                    'callback' => [$this, 'update_price'],
                    'permission_callback' => [$this, 'permissions_check'],
                ],
                'schema' => [$this, 'get_public_item_schema'],
            ]
        );

        register_rest_route(
            $this->namespace,
            '/product/mass-update-qty',
            [
                [
                    'args' => [
                        'data' => [
                            'description' => __('is an array like array(product_id=>qty)'),
                            'type' => 'object',
                            'required' => true,
                        ],
                    ],
                    'methods' => \WP_REST_Server::EDITABLE,
                    'callback' => [$this, 'mass_update_qty'],
                    'permission_callback' => [$this, 'permissions_check'],
                ],
                'schema' => [$this, 'get_public_item_schema'],
            ]
        );

        register_rest_route(
            $this->namespace,
            '/product/mass-update-price',
            [
                [
                    'args' => [
                        'data' => [
                            'description' => __('is an array like array(product_id=>qty)'),
                            'type' => 'object',
                            'required' => true,
                        ],
                    ],
                    'methods' => \WP_REST_Server::EDITABLE,
                    'callback' => [$this, 'mass_update_price'],
                    'permission_callback' => [$this, 'permissions_check'],
                ],
                'schema' => [$this, 'get_public_item_schema'],
            ]
        );
    }


    /**
     * Get a collection of items
     *
     * @param WP_REST_Request $request Full data about the request.
     * @return WP_Error|WP_REST_Response
     */
    public function get_orders($request)
    {
        $data = [];

        $params = [
            'posts_per_page' => 20,
            'paged' => 1,
            'paginate' => true,
            'last_id' => 0,
        ];

        if ($request->has_param('last_id')) {
            $params['last_id'] = (int)$request->get_param('last_id');
        }

        if ($request->has_param('page')) {
            $params['paged'] = $request->get_param('page');
        }

        if ($request->has_param('per_page')) {
            $params['posts_per_page'] = $request->get_param('per_page');
        }

        add_filter( 'woocommerce_orders_table_query_clauses', function ($q) use ($params){
            global $wpdb;


            $q['where'] .= ' and ' . $wpdb->prefix . 'wc_orders.id > ' . $params['last_id'];

            return $q;
        } );

        $order_query = new WC_Order_Query([
            'paginate' => true,
            'limit' => $params['posts_per_page'],
            'page' => $params['paged'],
        ]);

        $orderData = $order_query->get_orders();

        $data['total_current'] = count($orderData->orders);
        $data['total'] = $orderData->total;
        $data['last_page'] = $orderData->max_num_pages;

        $orders = [];

        foreach ($orderData->orders as $order) {

            $items = [];

            foreach ($order->get_items() as $item_id => $item) {
                $items[] = $this->buildFields([
                    'id' => $item_id,
                    'order_id' => $item->get_order_id(),
                    'product_id' => $item->get_product_id(),
                    'product_name' => $item->get_name(),
                    'quantity' => $item->get_quantity(),
                    'subtotal' => $item->get_subtotal(),
                    'total' => $item->get_total(),
                ], $this->getFields($request, 'items'));
            }

            $orderResponse = $this->buildFields([
                'id' => $order->id,
                'status' => $order->get_status(),
                'prices_include_tax' => $order->get_prices_include_tax(),
                'discount_total' => $order->get_discount_total(),
                'shipping_total' => $order->get_shipping_total(),
                'total' => $order->get_total(),
                'customer_id' => $order->customer_id ?? 0,
                'payment_method_title' => $order->payment_method_title,
                'payment_method' => $order->payment_method,
                'date_created' => $order->date_created,
            ], $this->getFields($request, 'order'));

            $orders[] = ['order' => $orderResponse, 'items' => $items];
        }

        $data['items'] = $orders;

        return new WP_REST_Response($data, 200);
    }

    /**
     * Get a collection of items
     *
     * @param WP_REST_Request $request Full data about the request.
     * @return WP_Error|WP_REST_Response
     */
    public function get_order($request)
    {
        $id = $request->get_param('order_id');

        $order = wc_get_order($id);

        if (!$order) {
            return new WP_Error(404, "order not found !");
        }

        $data = $order->get_data();

        return new WP_REST_Response($data, 200);
    }

    /**
     * Get a collection of items
     *
     * @param WP_REST_Request $request Full data about the request.
     * @return WP_Error|WP_REST_Response
     */
    public function get_customers($request)
    {
        $page = 1;
        $per_page = 20;
        $last_id = 0;

        if ($request->has_param('last_id')) {
            $last_id = (int)$request->get_param('last_id');
        }

        if ($request->has_param('page')) {
            $page = $request->get_param('page');
        }

        if ($request->has_param('per_page')) {
            $per_page = $request->get_param('per_page');
        }

        add_filter( 'pre_user_query', function ($q) use ($last_id){
            global $wpdb;

            $q->query_where.= ' and ' . $wpdb->prefix . 'users.id > ' . $last_id;

            return $q;
        } );

        $customer_query = new WP_User_Query([
            'count_total' => true,
            'fields' => 'id',
            'paged' => $page,
            'number' => $per_page,
            self::CHALAKHESAB_CHEKER_PARAM => 0,
            'role' => 'customer',
        ]);

        $customers = $customer_query->get_results();

        $items = [];

        foreach ($customers as $customer_id) {
            $customer = new WC_Customer($customer_id);
            $customer->update_meta_data(self::CHALAKHESAB_CHEKER_PARAM, 1);

            $data = $customer->get_data();

            $items[] = $this->buildFields($data, $this->getFields($request));
        }

        $data = [
            'items' => $items,
            'total_current' => count($items),
            'total' => $customer_query->get_total(),
            'last_page' => ceil($customer_query->get_total() / $per_page),
        ];

        return new WP_REST_Response($data, 200);
    }

    /**
     * Get a collection of items
     *
     * @param WP_REST_Request $request Full data about the request.
     * @return WP_Error|WP_REST_Response
     */
    public function get_customer($request)
    {
        $id = $request->get_param('customer_id');

        $customer = new WC_Customer($id);

        if (!$customer) {
            return new WP_Error(404, "customer not found !");
        }

        $data = $customer->get_data();

        return new WP_REST_Response($data, 200);
    }

    /**
     * Get a collection of items
     *
     * @param WP_REST_Request $request Full data about the request.
     * @return WP_Error|WP_REST_Response
     */
    public function get_product($request)
    {
        $product_id = $request->get_param('product_id');

        $data = wc_get_product($product_id)->get_data();

        return new WP_REST_Response($data, 200);
    }

    /**
     * Get a collection of items
     *
     * @param WP_REST_Request $request Full data about the request.
     * @return WP_Error|WP_REST_Response
     */
    public function get_products($request)
    {
        $params = [
            'posts_per_page' => 20,
            'paged' => 1,
            'paginate' => true,
            'last_id' => 0,
        ];

        if ($request->has_param('last_id')) {
            $params['last_id'] = (int)$request->get_param('last_id');
        }

        if ($request->has_param('page')) {
            $params['paged'] = $request->get_param('page');
        }

        if ($request->has_param('per_page')) {
            $params['posts_per_page'] = $request->get_param('per_page');
        }

        add_filter( 'posts_where', function ($q) use ($params){
            global $wpdb;

            return $q . ' and ' . $wpdb->prefix . 'posts.id > ' . $params['last_id'];
        } );

        $items = [];

        $productData = wc_get_products($params);

        foreach ($productData->products as $product) {

            $d = wc_get_product($product)->get_data();

            $ids = $d['category_ids'];

            foreach ($ids as $id) {
                $d['categories'][$id] = get_the_category_by_ID($id);
            }

            $items[] = $this->buildFields($d, $this->getFields($request));;
        }

        $data = [
            'items' => $items,
            'total_current' => count($productData->products),
            'total' => $productData->total,
            'last_page' => $productData->max_num_pages,
        ];

        return new WP_REST_Response($data, 200);
    }


    private function _update_qty($product_id, $qty)
    {
        wc_update_product_stock_status($product_id, $qty > 0 ? "instock" : "outofstock");

        $product = wc_get_product($product_id);

        $product->set_stock_quantity($qty);

        return $product->save();
    }

    /**
     * @param WP_REST_Request $request
     * @return void
     */
    public function update_qty($request)
    {
        $product_id = $request->get_param('product_id');

        $qty = $request->get_param('qty');

        $this->_update_qty($product_id, $qty);

        return new WP_REST_Response(['message' => 'product has been updated'], 200);
    }

    /**
     * @param WP_REST_Request $request
     * @return void
     */
    public function mass_update_qty($request)
    {
        $data = $request->get_param('data');

        foreach ($data as $product_id => $qty) {
            $this->_update_qty($product_id, $qty);
        }

        return new WP_REST_Response(['message' => 'product has been updated'], 200);
    }

    /**
     * @param WP_REST_Request $request
     * @return int
     */
    private function _update_price($product_id, $price)
    {

        $product = wc_get_product($product_id);

        $product->set_regular_price($price);

        return $product->save();
    }

    /**
     * @param WP_REST_Request $request
     * @return void
     */
    public function update_price($request)
    {
        $product_id = $request->get_param('product_id');

        $price = $request->get_param('price');

        $this->_update_price($product_id, $price);

        return new WP_REST_Response([
            'status' => true,
            'message' => 'product has been updated'
        ], 200);
    }

    /**
     * @param WP_REST_Request $request
     * @return void
     */
    public function mass_update_price($request)
    {
        $data = $request->get_param('data');

        foreach ($data as $productId => $price) {
            $this->_update_price($productId, $price);
        }

        return new WP_REST_Response([
            'status' => true,
            'message' => 'product has been updated'
        ], 200);
    }


    /**
     * Get the query params for collections
     *
     * @return array
     */
    public function get_collection_params()
    {
        return array(
            'page' => array(
                'description' => 'Current page of the collection.',
                'type' => 'integer',
                'default' => 1,
                'sanitize_callback' => 'absint',
            ),
            'per_page' => array(
                'description' => 'Maximum number of items to be returned in result set.',
                'type' => 'integer',
                'default' => 10,
                'sanitize_callback' => 'absint',
            ),
            'search' => array(
                'description' => 'Limit results to those matching a string.',
                'type' => 'string',
                'sanitize_callback' => 'sanitize_text_field',
            ),
        );
    }
}
